@@ -29,6 +29,7 @@
 from __future__ import absolute_import, division, print_function
 __metaclass__ = type
 
+import json
 import logging
 import os
 import pwd
@@ -42,7 +43,6 @@ import ansible.vars.clean
 
 from ansible.module_utils.common.text.converters import to_bytes, to_text
 from ansible.module_utils.six.moves import shlex_quote
-from ansible.parsing.utils.jsonify import jsonify
 
 import mitogen.core
 import mitogen.select
@@ -219,8 +219,13 @@ class ActionModuleMixin(ansible.plugins.action.ActionBase):
         Used by the base _execute_module(), and in <2.4 also by the template
         action module, and probably others.
         """
+        if data is None and ansible_mitogen.utils.ansible_version[:2] <= (2, 18):
+            data = '{}'
         if isinstance(data, dict):
-            data = jsonify(data)
+            try:
+                data = json.dumps(data, ensure_ascii=False)
+            except UnicodeDecodeError:
+                data = json.dumps(data)
         if not isinstance(data, bytes):
             data = to_bytes(data, errors='surrogate_or_strict')
 
@@ -402,15 +407,17 @@ class ActionModuleMixin(ansible.plugins.action.ActionBase):
             if not self._mitogen_rediscovered_interpreter:
                 result['ansible_facts'][self._discovered_interpreter_key] = self._discovered_interpreter
 
-        if self._discovery_warnings:
+        discovery_warnings = getattr(self, '_discovery_warnings', [])
+        if discovery_warnings:
             if result.get('warnings') is None:
                 result['warnings'] = []
-            result['warnings'].extend(self._discovery_warnings)
+            result['warnings'].extend(discovery_warnings)
 
-        if self._discovery_deprecation_warnings:
+        discovery_deprecation_warnings = getattr(self, '_discovery_deprecation_warnings', [])
+        if discovery_deprecation_warnings:
             if result.get('deprecations') is None:
                 result['deprecations'] = []
-            result['deprecations'].extend(self._discovery_deprecation_warnings)
+            result['deprecations'].extend(discovery_deprecation_warnings)
 
         return ansible.utils.unsafe_proxy.wrap_var(result)
 
@@ -429,7 +436,10 @@ class ActionModuleMixin(ansible.plugins.action.ActionBase):
                     "stderr": "stderr data"
                 }
         """
-        data = self._parse_returned_data(result)
+        if ansible_mitogen.utils.ansible_version[:2] >= (2, 19):
+            data = self._parse_returned_data(result, profile='legacy')
+        else:
+            data = self._parse_returned_data(result)
 
         # Cutpasted from the base implementation.
         if 'stdout' in data and 'stdout_lines' not in data:
@@ -460,18 +470,7 @@ class ActionModuleMixin(ansible.plugins.action.ActionBase):
         # chicken-and-egg issue, mitogen needs a python to run low_level_execute_command
         # which is required by Ansible's discover_interpreter function
         if self._mitogen_discovering_interpreter:
-            possible_pythons = [
-                '/usr/bin/python',
-                'python3',
-                'python3.7',
-                'python3.6',
-                'python3.5',
-                'python2.7',
-                'python2.6',
-                '/usr/libexec/platform-python',
-                '/usr/bin/python3',
-                'python'
-            ]
+            possible_pythons = self._mitogen_interpreter_candidates
         else:
             # not used, just adding a filler value
             possible_pythons = ['python']
@@ -482,12 +481,15 @@ class ActionModuleMixin(ansible.plugins.action.ActionBase):
                 rc, stdout, stderr = self._connection.exec_command(
                     cmd, in_data, sudoable, mitogen_chdir=chdir,
                 )
-            # TODO: what exception is thrown?
-            except:
+            except BaseException as exc:
                 # we've reached the last python attempted and failed
                 if possible_python == possible_pythons[-1]:
                     raise
                 else:
+                    LOG.debug(
+                        '%r._low_level_execute_command: candidate=%r ignored: %s, %r',
+                        self, possible_python, type(exc), exc,
+                    )
                     continue
 
         stdout_text = to_text(stdout, errors=encoding_errors)
